// @ts-check
import axios from 'axios';
import qs from 'qs';

const FILE_UPLOAD_CHUNK_SIZE = 1048576; // 1024 * 1024 = 1 MiB

export default class UploadHandler {
    /**
     * @param {JQuery<HTMLElement>} $file File Input
     * @param {JQuery<HTMLElement>} $form form
     * @param {JQuery<HTMLElement>} $progress Progress Bar
     */
    constructor($file, $form, $progress) {
        /** @type {?File} */
        this.selectedFile = null;

        this.hasErros = false;
        this.$file = $file;
        this.$form = $form;
        this.$progress = $progress;

        this.config = {
            method: this.$form.attr('method'),
            headers: {'content-type': 'application/x-www-form-urlencoded'},
            url: this.$form.attr('action')
        };

        this.registerEvents();
    }

    /**
     * Register Event handler.
     */
    registerEvents() {
        this.$file.on('change', this.onUploadFileChange.bind(this));
        this.$form.on('submit', this.onFormSubmit.bind(this));
    }

    /**
     * File Change Event Handler.
     * @param {Event} event
     */
    onUploadFileChange(event) {
        this.selectedFile = event.target.files[0] || null;
    }

    /**
     * Form Submit Event Handler takes care of the file upload
     * @param {Event} event
     */
    async onFormSubmit(event) {
        event.preventDefault();

        if (this.selectedFile) {
            // Start Progress
            this.hasErros = false;
            this.$progress.show();
            this.fullUploadProgess(0);

            // Handle Chunk Upload
            await this.uploadFullBackupFile();
            this.$file.val('');

            // Done
            this.fullUploadProgess(0);
            this.$progress.hide();
            this.$form.find('[name="fileName"]').val(this.selectedFile.name);
            this.selectedFile = null;

            if (!this.hasErros) {
                this.$form.off('submit').trigger('submit');
            }
        }
    }

    // This uploader uses the JS FileReader API because backups can be very big, see https://deliciousbrains.com/using-javascript-file-api-to-avoid-file-upload-limits/
    async uploadFullBackupFile() {
        const reader = new FileReader();

        let offset = 0;
        let complete = false;
        let isFirst = true;

        while (!complete) {
            let nextChunk = offset + FILE_UPLOAD_CHUNK_SIZE + 1;
            let currentChunk = this.selectedFile.slice(offset, nextChunk);
            // complete = await this._uploadFullBackupFileChunk(file, reader, offset, currentChunk, nextChunk. applyNow, progressCallback, this.config);
            try {
                complete = await new Promise((resolve, reject) => {
                    // register for the loaded event of the file reader
                    reader.onloadend = async function (event) {
                        if (event.target.readyState !== FileReader.DONE) {
                            return;
                        }

                        // Perform request for the current chunk
                        let isFinal = (nextChunk >= this.selectedFile.size);
                        let requestConfig = this.config;
                        requestConfig.data = qs.stringify({
                            csrfToken: this.$form.find('.jtl_token').val(),
                            chunk: event.target.result,
                            fileName: this.selectedFile.name,
                            fileType: this.selectedFile.type,
                            fileSize: this.selectedFile.size,
                            action: 'uploadFullBackupFileChunk',
                            isFirst: isFirst,
                            isFinal: isFinal,
                            isAjax: true
                        });

                        try {
                            let result = await axios.request(requestConfig);
                            // Check success
                            if(!result || !result.data || !result.data.data || result.data.data !== true) {
                                // Failed :/
                                reject(result.data);
                            }
                            let sizeDone = offset + FILE_UPLOAD_CHUNK_SIZE;
                            let percentDone = Math.min(100, Math.floor((sizeDone / this.selectedFile.size) * 100));
                            this.fullUploadProgess(percentDone);
                            resolve(isFinal);
                        } catch (e) {
                            console.log(e);
                            reject(e);
                        }
                    }.bind(this);

                    // We use readAsDataURL to prevent encoding problems. (This encodes the data into base64.)
                    // This will trigger the event registered above.
                    reader.readAsDataURL(currentChunk);
                });

            } catch (err) {
                this.hasErros = true;
                console.error(err);
                this.$form.prepend(
                    `<div class="alert alert-danger" role="alert">
                        ${err.message}
                    </div>`);
                break;
            }

            isFirst = false;
            if(!complete) {
                offset = nextChunk;
            }
        }

        return true;
    }

    fullUploadProgess(percentage) {
        this.$progress.show();
        this.$progress.find('.progress-bar').width(percentage + '%').attr('aria-valuenow', percentage);
    }
}